/*
 Photo Wizard
 A batch image convertor that performs simple image processing tasks.
 Copyright  2004, Manas Tungare.
 --------------------------------------------------------------------------
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.Text;

namespace Org.ManasTungare.PhotoWizard.Operations
{
	/// <summary>
	/// Increase or decrease brightness of the image.
	/// </summary>
	public class Brightness : System.Windows.Forms.UserControl, Org.ManasTungare.PhotoWizard.IOperation
	{
		private System.Windows.Forms.GroupBox brightnessGroupBox;
		private System.Windows.Forms.CheckBox enableBrightnessCheckBox;
		private System.Windows.Forms.TrackBar brightnessTrackBar;
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public Brightness()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.brightnessGroupBox = new System.Windows.Forms.GroupBox();
			this.brightnessTrackBar = new System.Windows.Forms.TrackBar();
			this.enableBrightnessCheckBox = new System.Windows.Forms.CheckBox();
			this.brightnessGroupBox.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.brightnessTrackBar)).BeginInit();
			this.SuspendLayout();
			// 
			// brightnessGroupBox
			// 
			this.brightnessGroupBox.Controls.Add(this.brightnessTrackBar);
			this.brightnessGroupBox.Controls.Add(this.enableBrightnessCheckBox);
			this.brightnessGroupBox.Dock = System.Windows.Forms.DockStyle.Fill;
			this.brightnessGroupBox.Location = new System.Drawing.Point(0, 0);
			this.brightnessGroupBox.Name = "brightnessGroupBox";
			this.brightnessGroupBox.Size = new System.Drawing.Size(288, 88);
			this.brightnessGroupBox.TabIndex = 0;
			this.brightnessGroupBox.TabStop = false;
			// 
			// brightnessTrackBar
			// 
			this.brightnessTrackBar.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.brightnessTrackBar.Enabled = false;
			this.brightnessTrackBar.LargeChange = 1;
			this.brightnessTrackBar.Location = new System.Drawing.Point(32, 40);
			this.brightnessTrackBar.Name = "brightnessTrackBar";
			this.brightnessTrackBar.Size = new System.Drawing.Size(240, 42);
			this.brightnessTrackBar.TabIndex = 1;
			this.brightnessTrackBar.Value = 5;
			// 
			// enableBrightnessCheckBox
			// 
			this.enableBrightnessCheckBox.Font = new System.Drawing.Font("Tahoma", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.enableBrightnessCheckBox.Location = new System.Drawing.Point(16, 16);
			this.enableBrightnessCheckBox.Name = "enableBrightnessCheckBox";
			this.enableBrightnessCheckBox.Size = new System.Drawing.Size(88, 24);
			this.enableBrightnessCheckBox.TabIndex = 0;
			this.enableBrightnessCheckBox.Text = "Brightness";
			this.enableBrightnessCheckBox.CheckedChanged += new System.EventHandler(this.enableBrightnessCheckBox_CheckedChanged);
			// 
			// Brightness
			// 
			this.AutoScroll = true;
			this.AutoScrollMinSize = new System.Drawing.Size(236, 88);
			this.Controls.Add(this.brightnessGroupBox);
			this.Font = new System.Drawing.Font("Tahoma", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.Name = "Brightness";
			this.Size = new System.Drawing.Size(288, 88);
			this.brightnessGroupBox.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.brightnessTrackBar)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		public string getImageMagickCommand() 
		{
			int brightnessPercent = 100 * this.brightnessTrackBar.Value / (this.brightnessTrackBar.Maximum / 2);
			return this.enableBrightnessCheckBox.Checked ? "-modulate " + brightnessPercent : "";
		}

		private void enableBrightnessCheckBox_CheckedChanged(object sender, System.EventArgs e)
		{
			this.brightnessTrackBar.Enabled = this.enableBrightnessCheckBox.Checked;
		}

	}
}
