/*
 Photo Wizard
 A batch image convertor that performs simple image processing tasks.
 Copyright  2004, Manas Tungare.
 --------------------------------------------------------------------------
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Text;
using System.IO;

namespace Org.ManasTungare.PhotoWizard
{
	/// <summary>
	/// Main window of the application; it also creates ImageMagick processes to do the
	/// actual work.
	/// </summary>
	public class PhotoWizard : System.Windows.Forms.Form
	{
		private System.Windows.Forms.StatusBar statusBar;
		private System.Windows.Forms.StatusBarPanel statusPanel;
		private Org.ManasTungare.PhotoWizard.Operations.Resizer resizer1;
		private Org.ManasTungare.PhotoWizard.Operations.Antialiaser antialiaser1;
		private Org.ManasTungare.PhotoWizard.Operations.Borderer borderer1;
		private Org.ManasTungare.PhotoWizard.Operations.Brightness brightness1;
		private Org.ManasTungare.PhotoWizard.Operations.Contrast contrast1;
		private Org.ManasTungare.PhotoWizard.Operations.Quality quality1;
		private Org.ManasTungare.PhotoWizard.Operations.Rotater rotater1;
		private System.Windows.Forms.ToolBar toolBar;
		private System.Windows.Forms.ToolBarButton processImagesButton;
		private System.Windows.Forms.Panel operationsPanel;
		private System.Windows.Forms.ListView fileList;
		private System.Windows.Forms.ToolBarButton addFilesButton;
		private System.Windows.Forms.ToolBarButton addDirectoryButton;
		private System.Windows.Forms.ColumnHeader fileColumnHeader;
		private System.Windows.Forms.ImageList imageList;
		private System.Windows.Forms.ImageList toolbarImageList;
		private System.Windows.Forms.MainMenu mainMenu;
		private System.Windows.Forms.MenuItem menuFileExit;
		private System.Windows.Forms.MenuItem menuFileAddFiles;
		private System.Windows.Forms.MenuItem menuFileAddDirectory;
		private System.Windows.Forms.MenuItem menuFile;
		private System.Windows.Forms.MenuItem menuProcess;
		private System.Windows.Forms.MenuItem menuProcessStart;
		private System.Windows.Forms.MenuItem menuHelp;
		private System.Windows.Forms.MenuItem menuHelpAboutPhotoWizard;
		private System.Windows.Forms.MenuItem menuSeparator1;
		private System.ComponentModel.IContainer components;

		private const string IMAGE_FILES_FILTER = ".png .jpg .jpeg .gif";

		public PhotoWizard()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(PhotoWizard));
			this.toolBar = new System.Windows.Forms.ToolBar();
			this.addFilesButton = new System.Windows.Forms.ToolBarButton();
			this.addDirectoryButton = new System.Windows.Forms.ToolBarButton();
			this.processImagesButton = new System.Windows.Forms.ToolBarButton();
			this.toolbarImageList = new System.Windows.Forms.ImageList(this.components);
			this.imageList = new System.Windows.Forms.ImageList(this.components);
			this.statusBar = new System.Windows.Forms.StatusBar();
			this.statusPanel = new System.Windows.Forms.StatusBarPanel();
			this.operationsPanel = new System.Windows.Forms.Panel();
			this.antialiaser1 = new Org.ManasTungare.PhotoWizard.Operations.Antialiaser();
			this.quality1 = new Org.ManasTungare.PhotoWizard.Operations.Quality();
			this.contrast1 = new Org.ManasTungare.PhotoWizard.Operations.Contrast();
			this.brightness1 = new Org.ManasTungare.PhotoWizard.Operations.Brightness();
			this.borderer1 = new Org.ManasTungare.PhotoWizard.Operations.Borderer();
			this.rotater1 = new Org.ManasTungare.PhotoWizard.Operations.Rotater();
			this.resizer1 = new Org.ManasTungare.PhotoWizard.Operations.Resizer();
			this.fileList = new System.Windows.Forms.ListView();
			this.fileColumnHeader = new System.Windows.Forms.ColumnHeader();
			this.mainMenu = new System.Windows.Forms.MainMenu();
			this.menuFile = new System.Windows.Forms.MenuItem();
			this.menuFileAddFiles = new System.Windows.Forms.MenuItem();
			this.menuFileAddDirectory = new System.Windows.Forms.MenuItem();
			this.menuSeparator1 = new System.Windows.Forms.MenuItem();
			this.menuFileExit = new System.Windows.Forms.MenuItem();
			this.menuProcess = new System.Windows.Forms.MenuItem();
			this.menuProcessStart = new System.Windows.Forms.MenuItem();
			this.menuHelp = new System.Windows.Forms.MenuItem();
			this.menuHelpAboutPhotoWizard = new System.Windows.Forms.MenuItem();
			((System.ComponentModel.ISupportInitialize)(this.statusPanel)).BeginInit();
			this.operationsPanel.SuspendLayout();
			this.SuspendLayout();
			// 
			// toolBar
			// 
			this.toolBar.Appearance = System.Windows.Forms.ToolBarAppearance.Flat;
			this.toolBar.Buttons.AddRange(new System.Windows.Forms.ToolBarButton[] {
																					   this.addFilesButton,
																					   this.addDirectoryButton,
																					   this.processImagesButton});
			this.toolBar.DropDownArrows = true;
			this.toolBar.ImageList = this.toolbarImageList;
			this.toolBar.Location = new System.Drawing.Point(0, 0);
			this.toolBar.Name = "toolBar";
			this.toolBar.ShowToolTips = true;
			this.toolBar.Size = new System.Drawing.Size(864, 44);
			this.toolBar.TabIndex = 0;
			this.toolBar.TextAlign = System.Windows.Forms.ToolBarTextAlign.Right;
			this.toolBar.ButtonClick += new System.Windows.Forms.ToolBarButtonClickEventHandler(this.toolBar_ButtonClick);
			// 
			// addFilesButton
			// 
			this.addFilesButton.ImageIndex = 0;
			this.addFilesButton.Text = "Add Files";
			// 
			// addDirectoryButton
			// 
			this.addDirectoryButton.ImageIndex = 1;
			this.addDirectoryButton.Text = "Add Directory";
			// 
			// processImagesButton
			// 
			this.processImagesButton.ImageIndex = 2;
			this.processImagesButton.Text = "Process";
			// 
			// toolbarImageList
			// 
			this.toolbarImageList.ColorDepth = System.Windows.Forms.ColorDepth.Depth32Bit;
			this.toolbarImageList.ImageSize = new System.Drawing.Size(32, 32);
			this.toolbarImageList.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("toolbarImageList.ImageStream")));
			this.toolbarImageList.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// imageList
			// 
			this.imageList.ColorDepth = System.Windows.Forms.ColorDepth.Depth32Bit;
			this.imageList.ImageSize = new System.Drawing.Size(16, 16);
			this.imageList.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageList.ImageStream")));
			this.imageList.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// statusBar
			// 
			this.statusBar.Location = new System.Drawing.Point(0, 743);
			this.statusBar.Name = "statusBar";
			this.statusBar.Panels.AddRange(new System.Windows.Forms.StatusBarPanel[] {
																						 this.statusPanel});
			this.statusBar.ShowPanels = true;
			this.statusBar.Size = new System.Drawing.Size(864, 22);
			this.statusBar.TabIndex = 1;
			// 
			// statusPanel
			// 
			this.statusPanel.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Spring;
			this.statusPanel.Text = "Welcome to Photo Wizard";
			this.statusPanel.Width = 848;
			// 
			// operationsPanel
			// 
			this.operationsPanel.AutoScroll = true;
			this.operationsPanel.Controls.Add(this.antialiaser1);
			this.operationsPanel.Controls.Add(this.quality1);
			this.operationsPanel.Controls.Add(this.contrast1);
			this.operationsPanel.Controls.Add(this.brightness1);
			this.operationsPanel.Controls.Add(this.borderer1);
			this.operationsPanel.Controls.Add(this.rotater1);
			this.operationsPanel.Controls.Add(this.resizer1);
			this.operationsPanel.Dock = System.Windows.Forms.DockStyle.Left;
			this.operationsPanel.DockPadding.Left = 2;
			this.operationsPanel.DockPadding.Right = 2;
			this.operationsPanel.Location = new System.Drawing.Point(0, 44);
			this.operationsPanel.Name = "operationsPanel";
			this.operationsPanel.Size = new System.Drawing.Size(288, 699);
			this.operationsPanel.TabIndex = 2;
			// 
			// antialiaser1
			// 
			this.antialiaser1.AutoScroll = true;
			this.antialiaser1.AutoScrollMargin = new System.Drawing.Size(4, 4);
			this.antialiaser1.AutoScrollMinSize = new System.Drawing.Size(0, 44);
			this.antialiaser1.Dock = System.Windows.Forms.DockStyle.Top;
			this.antialiaser1.Font = new System.Drawing.Font("Tahoma", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.antialiaser1.Location = new System.Drawing.Point(2, 724);
			this.antialiaser1.Name = "antialiaser1";
			this.antialiaser1.Size = new System.Drawing.Size(268, 48);
			this.antialiaser1.TabIndex = 1;
			// 
			// quality1
			// 
			this.quality1.AutoScroll = true;
			this.quality1.AutoScrollMinSize = new System.Drawing.Size(236, 88);
			this.quality1.Dock = System.Windows.Forms.DockStyle.Top;
			this.quality1.Font = new System.Drawing.Font("Tahoma", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.quality1.Location = new System.Drawing.Point(2, 636);
			this.quality1.Name = "quality1";
			this.quality1.Size = new System.Drawing.Size(268, 88);
			this.quality1.TabIndex = 5;
			// 
			// contrast1
			// 
			this.contrast1.AutoScroll = true;
			this.contrast1.AutoScrollMinSize = new System.Drawing.Size(236, 88);
			this.contrast1.Dock = System.Windows.Forms.DockStyle.Top;
			this.contrast1.Font = new System.Drawing.Font("Tahoma", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.contrast1.Location = new System.Drawing.Point(2, 548);
			this.contrast1.Name = "contrast1";
			this.contrast1.Size = new System.Drawing.Size(268, 88);
			this.contrast1.TabIndex = 4;
			// 
			// brightness1
			// 
			this.brightness1.AutoScroll = true;
			this.brightness1.AutoScrollMinSize = new System.Drawing.Size(236, 88);
			this.brightness1.Dock = System.Windows.Forms.DockStyle.Top;
			this.brightness1.Font = new System.Drawing.Font("Tahoma", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.brightness1.Location = new System.Drawing.Point(2, 460);
			this.brightness1.Name = "brightness1";
			this.brightness1.Size = new System.Drawing.Size(268, 88);
			this.brightness1.TabIndex = 3;
			// 
			// borderer1
			// 
			this.borderer1.AutoScroll = true;
			this.borderer1.AutoScrollMargin = new System.Drawing.Size(4, 4);
			this.borderer1.AutoScrollMinSize = new System.Drawing.Size(232, 164);
			this.borderer1.Dock = System.Windows.Forms.DockStyle.Top;
			this.borderer1.Font = new System.Drawing.Font("Tahoma", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.borderer1.Location = new System.Drawing.Point(2, 292);
			this.borderer1.Name = "borderer1";
			this.borderer1.Size = new System.Drawing.Size(268, 168);
			this.borderer1.TabIndex = 2;
			// 
			// rotater1
			// 
			this.rotater1.AutoScroll = true;
			this.rotater1.AutoScrollMargin = new System.Drawing.Size(4, 4);
			this.rotater1.AutoScrollMinSize = new System.Drawing.Size(240, 120);
			this.rotater1.Dock = System.Windows.Forms.DockStyle.Top;
			this.rotater1.Font = new System.Drawing.Font("Tahoma", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.rotater1.Location = new System.Drawing.Point(2, 164);
			this.rotater1.Name = "rotater1";
			this.rotater1.Size = new System.Drawing.Size(268, 128);
			this.rotater1.TabIndex = 6;
			// 
			// resizer1
			// 
			this.resizer1.AutoScroll = true;
			this.resizer1.AutoScrollMargin = new System.Drawing.Size(4, 4);
			this.resizer1.AutoScrollMinSize = new System.Drawing.Size(232, 144);
			this.resizer1.Dock = System.Windows.Forms.DockStyle.Top;
			this.resizer1.Font = new System.Drawing.Font("Tahoma", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.resizer1.Location = new System.Drawing.Point(2, 0);
			this.resizer1.Name = "resizer1";
			this.resizer1.Size = new System.Drawing.Size(268, 164);
			this.resizer1.TabIndex = 0;
			// 
			// fileList
			// 
			this.fileList.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																					   this.fileColumnHeader});
			this.fileList.Dock = System.Windows.Forms.DockStyle.Fill;
			this.fileList.FullRowSelect = true;
			this.fileList.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Nonclickable;
			this.fileList.Location = new System.Drawing.Point(288, 44);
			this.fileList.MultiSelect = false;
			this.fileList.Name = "fileList";
			this.fileList.Size = new System.Drawing.Size(576, 699);
			this.fileList.SmallImageList = this.imageList;
			this.fileList.TabIndex = 4;
			this.fileList.View = System.Windows.Forms.View.Details;
			// 
			// fileColumnHeader
			// 
			this.fileColumnHeader.Text = "File";
			this.fileColumnHeader.Width = 480;
			// 
			// mainMenu
			// 
			this.mainMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					 this.menuFile,
																					 this.menuProcess,
																					 this.menuHelp});
			// 
			// menuFile
			// 
			this.menuFile.Index = 0;
			this.menuFile.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					 this.menuFileAddFiles,
																					 this.menuFileAddDirectory,
																					 this.menuSeparator1,
																					 this.menuFileExit});
			this.menuFile.Text = "&File";
			// 
			// menuFileAddFiles
			// 
			this.menuFileAddFiles.Index = 0;
			this.menuFileAddFiles.Shortcut = System.Windows.Forms.Shortcut.CtrlF;
			this.menuFileAddFiles.Text = "&Add Files ...";
			this.menuFileAddFiles.Click += new System.EventHandler(this.menuFileAddFiles_Click);
			// 
			// menuFileAddDirectory
			// 
			this.menuFileAddDirectory.Index = 1;
			this.menuFileAddDirectory.Shortcut = System.Windows.Forms.Shortcut.CtrlD;
			this.menuFileAddDirectory.Text = "Ad&d Directory ...";
			this.menuFileAddDirectory.Click += new System.EventHandler(this.menuFileAddDirectory_Click);
			// 
			// menuSeparator1
			// 
			this.menuSeparator1.Index = 2;
			this.menuSeparator1.Text = "-";
			// 
			// menuFileExit
			// 
			this.menuFileExit.Index = 3;
			this.menuFileExit.Shortcut = System.Windows.Forms.Shortcut.AltF4;
			this.menuFileExit.Text = "E&xit";
			this.menuFileExit.Click += new System.EventHandler(this.menuFileExit_Click);
			// 
			// menuProcess
			// 
			this.menuProcess.Index = 1;
			this.menuProcess.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																						this.menuProcessStart});
			this.menuProcess.Text = "&Process";
			// 
			// menuProcessStart
			// 
			this.menuProcessStart.Index = 0;
			this.menuProcessStart.Shortcut = System.Windows.Forms.Shortcut.CtrlP;
			this.menuProcessStart.Text = "&Start";
			this.menuProcessStart.Click += new System.EventHandler(this.menuProcessStart_Click);
			// 
			// menuHelp
			// 
			this.menuHelp.Index = 2;
			this.menuHelp.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					 this.menuHelpAboutPhotoWizard});
			this.menuHelp.Text = "&Help";
			// 
			// menuHelpAboutPhotoWizard
			// 
			this.menuHelpAboutPhotoWizard.Index = 0;
			this.menuHelpAboutPhotoWizard.Text = "&About ...";
			this.menuHelpAboutPhotoWizard.Click += new System.EventHandler(this.menuHelpAboutPhotoWizard_Click);
			// 
			// PhotoWizard
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 14);
			this.ClientSize = new System.Drawing.Size(864, 765);
			this.Controls.Add(this.fileList);
			this.Controls.Add(this.operationsPanel);
			this.Controls.Add(this.statusBar);
			this.Controls.Add(this.toolBar);
			this.Font = new System.Drawing.Font("Tahoma", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Menu = this.mainMenu;
			this.Name = "PhotoWizard";
			this.Text = "Photo Wizard";
			((System.ComponentModel.ISupportInitialize)(this.statusPanel)).EndInit();
			this.operationsPanel.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		private void addFilesToListView() 
		{
			OpenFileDialog fileSelectDialog = new OpenFileDialog();
			fileSelectDialog.AddExtension = false;
			fileSelectDialog.CheckFileExists = true;
			fileSelectDialog.CheckPathExists = true;
			fileSelectDialog.Filter = "Image Files (PNG, JPEG, GIF)|*.png;*.jpg;*.jpeg;*.gif";
			fileSelectDialog.Multiselect = true;
			fileSelectDialog.Title = "Pick Files";
			fileSelectDialog.ShowDialog();

			foreach (string selectedFile in fileSelectDialog.FileNames) 
			{
				this.fileList.Items.Add (new ListViewItem(selectedFile, 0));
			}
		}

		private void addFolderToListView() 
		{
			FolderBrowserDialog folderBrowserDialog = new FolderBrowserDialog();
			folderBrowserDialog.ShowDialog();

			if (folderBrowserDialog.SelectedPath.Length > 0) 
			{
				addFilesRecursively(new DirectoryInfo(folderBrowserDialog.SelectedPath), IMAGE_FILES_FILTER);
			}
		}

		private void addFilesRecursively(DirectoryInfo directory, string filter) 
		{
			foreach(FileInfo imageFile in directory.GetFiles() ) 
			{
				if (imageFile.Extension.Length > 0 && filter.IndexOf(imageFile.Extension.ToLower()) > -1) 
				{
					this.fileList.Items.Add(new ListViewItem(imageFile.FullName, 0));
				}
			}

			foreach (DirectoryInfo subDirectory in directory.GetDirectories() ) 
			{
				addFilesRecursively(subDirectory, filter);
			}
		}


		private void startProcessing() 
		{
			if (this.fileList.Items.Count > 0) 
			{
				if(MessageBox.Show("This will process each image and overwrite the existing one. Do you want to continue?", "Overwrite Images?",  MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.Yes) 
				{
					doProcessing();
				}
			}
			else
			{
				MessageBox.Show("No files to process! Please add files or folders before attempting to process them.");
			}
		}

		private void doProcessing()
		{
			// Create command line from individual operations
			StringBuilder output = new StringBuilder();
			foreach (Control control in this.operationsPanel.Controls) 
			{
				if (control is IOperation) 
				{
					output.Append ( ((IOperation) control).getImageMagickCommand() );
					output.Append (" ");
				}
			}
			string commands = output.ToString();

			// Create Status Window and display it
			StatusWindow processingStatus = new StatusWindow();
			processingStatus.Show();
			processingStatus.BringToFront();

			// Initialize status counter
			int fileCount = 0;

			// Shell out to the ImageMagick binary to process given files
			foreach (ListViewItem fileItem in this.fileList.Items) 
			{
				fileCount++;
				processingStatus.CurrentFile = fileItem.Text;
				processingStatus.StatusMessage = "Processing " + fileCount + " of " + this.fileList.Items.Count + " files ...";
				Application.DoEvents();

				System.Diagnostics.Process proc = new System.Diagnostics.Process();
				proc.EnableRaisingEvents = true;
				proc.StartInfo.FileName = "convert.exe";
				proc.StartInfo.CreateNoWindow = true;
				proc.StartInfo.Arguments = commands + " \"" + fileItem.Text + "\" \"" + fileItem.Text + "\"";
				proc.StartInfo.UseShellExecute = false;
				proc.StartInfo.RedirectStandardError = true;
				proc.StartInfo.RedirectStandardOutput = true;
				proc.Start();

				while (! proc.HasExited ) 
				{
					// Busy waiting, not the best approach.
					// Need to read more about the System.Diagnostics.Process class.
					Application.DoEvents();

					if (processingStatus.CancelPressed) 
					{
						proc.Kill();
						break;
					}
				}

				string errorMessage = proc.StandardError.ReadToEnd();
				if (errorMessage.Length > 0) {
					MessageBox.Show("Error: " + errorMessage);
				}

				if (processingStatus.CancelPressed) 
				{
					break;
				}
			}

			processingStatus.Hide();
		}

		private void toolBar_ButtonClick(object sender, System.Windows.Forms.ToolBarButtonClickEventArgs e)
		{
			if (e.Button == this.addFilesButton) 
			{
				addFilesToListView();		
			}
			else if (e.Button == this.addDirectoryButton) 
			{
				addFolderToListView();
			}
			else if (e.Button == this.processImagesButton) 
			{
				startProcessing();	
			}
		}

		private void menuFileAddFiles_Click(object sender, System.EventArgs e)
		{
			addFilesToListView();
		}

		private void menuHelpAboutPhotoWizard_Click(object sender, System.EventArgs e)
		{
			(new About()).ShowDialog();
		}

		private void menuFileExit_Click(object sender, System.EventArgs e)
		{
			EntryPoint.Shutdown();
		}

		private void menuFileAddDirectory_Click(object sender, System.EventArgs e)
		{
			addFolderToListView();		
		}

		private void menuProcessStart_Click(object sender, System.EventArgs e)
		{
			startProcessing();
		}
	}
}
